@@ -12,6 +12,7 @@ import (
 	"github.com/icinga/icinga-go-library/retry"
 	"github.com/icinga/icinga-go-library/types"
 	"github.com/icinga/icinga-go-library/utils"
+	"github.com/icinga/icingadb/internal"
 	v1 "github.com/icinga/icingadb/pkg/icingadb/v1"
 	"github.com/icinga/icingadb/pkg/icingaredis"
 	icingaredisv1 "github.com/icinga/icingadb/pkg/icingaredis/v1"
@@ -146,24 +147,13 @@ func (h *HA) controller() {
 
 	oldInstancesRemoved := false
 
-	// Suppress recurring log messages in the realize method to be only logged this often.
+	// Suppress recurring log messages in the realize method to be logged at INFO level every time.
 	routineLogTicker := time.NewTicker(5 * time.Minute)
 	defer routineLogTicker.Stop()
-	shouldLogRoutineEvents := true
-
-	// The retry logic in HA is twofold:
-	//
-	// 1) Updating or inserting the instance row based on the current heartbeat must be done within the heartbeat's
-	//    expiration time. Therefore, we use a deadline ctx to retry.WithBackoff() in realize() which expires earlier
-	//    than our default timeout.
-	// 2) Since we do not want to exit before our default timeout expires, we have to repeat step 1 until it does.
-	retryTimeout := time.NewTimer(retry.DefaultTimeout)
-	defer retryTimeout.Stop()
+	infoLogRoutineEvents := true
 
 	for {
 		select {
-		case <-retryTimeout.C:
-			h.abort(errors.New("retry deadline exceeded"))
 		case m := <-h.heartbeat.Events():
 			if m != nil {
 				now := time.Now()
@@ -181,9 +171,6 @@ func (h *HA) controller() {
 					h.signalHandover("received heartbeat from the past")
 					h.realizeLostHeartbeat()
 
-					// Reset retry timeout so that the next iterations have the full amount of time available again.
-					retry.ResetTimeout(retryTimeout, retry.DefaultTimeout)
-
 					continue
 				}
 				s, err := m.Stats().IcingaStatus()
@@ -215,21 +202,20 @@ func (h *HA) controller() {
 
 				select {
 				case <-routineLogTicker.C:
-					shouldLogRoutineEvents = true
+					infoLogRoutineEvents = true
 				default:
 				}
 
 				// Ensure that updating/inserting the instance row is completed by the current heartbeat's expiry time.
 				realizeCtx, cancelRealizeCtx := context.WithDeadline(h.ctx, m.ExpiryTime())
-				err = h.realize(realizeCtx, s, envId, shouldLogRoutineEvents)
+				err = h.realize(realizeCtx, s, envId, infoLogRoutineEvents)
 				cancelRealizeCtx()
 				if errors.Is(err, context.DeadlineExceeded) {
+					h.logger.Warnw("Instance update/insert exceeded heartbeat expiration time", zap.Error(err))
 					h.signalHandover("instance update/insert deadline exceeded heartbeat expiry time")
 
 					// Instance insert/update was not completed by the expiration time of the current heartbeat.
-					// Pass control back to the loop to try again with the next heartbeat,
-					// or exit the loop when the retry timeout has expired. Therefore,
-					// retry timeout is **not** reset here so that retries continue until the timeout has expired.
+					// Pass control back to the loop to try again with the next heartbeat.
 					continue
 				}
 				if err != nil {
@@ -241,20 +227,12 @@ func (h *HA) controller() {
 					oldInstancesRemoved = true
 				}
 
-				shouldLogRoutineEvents = false
+				infoLogRoutineEvents = false
 			} else {
 				h.logger.Error("Lost heartbeat")
 				h.signalHandover("lost heartbeat")
 				h.realizeLostHeartbeat()
 			}
-
-			// Reset retry timeout so that the next iterations have the full amount of time available again.
-			// Don't be surprised by the location of the code,
-			// as it is obvious that the timer is also reset after an error that ends the loop anyway.
-			// But this is the best place to catch all scenarios where the timeout needs to be reset.
-			// And since HA needs quite a bit of refactoring anyway to e.g. return immediately after calling h.abort(),
-			// it's fine to have it here for now.
-			retry.ResetTimeout(retryTimeout, retry.DefaultTimeout)
 		case <-h.heartbeat.Done():
 			if err := h.heartbeat.Err(); err != nil {
 				h.abort(err)
@@ -270,7 +248,7 @@ func (h *HA) controller() {
 // The context passed is expected to have a deadline, otherwise the method will panic. This deadline is strictly
 // enforced to abort the realization logic the moment the context expires.
 //
-// shouldLogRoutineEvents indicates if recurrent events should be logged.
+// infoLogRoutineEvents indicates if recurring events should be logged at "info" or "debug" level.
 //
 // The internal, retryable function always fetches the last received heartbeat's timestamp instead of reusing the one
 // from the calling controller loop. Doing so results in inserting a more accurate timestamp if a retry happens.
@@ -278,7 +256,7 @@ func (h *HA) realize(
 	ctx context.Context,
 	s *icingaredisv1.IcingaStatus,
 	envId types.Binary,
-	shouldLogRoutineEvents bool,
+	infoLogRoutineEvents bool,
 ) error {
 	var (
 		takeover         string
@@ -289,6 +267,11 @@ func (h *HA) realize(
 		panic("can't use context w/o deadline in realize()")
 	}
 
+	routineEventsLogLevel := zap.DebugLevel
+	if infoLogRoutineEvents {
+		routineEventsLogLevel = zap.InfoLevel
+	}
+
 	err := retry.WithBackoff(
 		ctx,
 		func(ctx context.Context) error {
@@ -322,6 +305,9 @@ func (h *HA) realize(
 			instance := &v1.IcingadbInstance{}
 			errQuery := tx.QueryRowxContext(ctx, query, envId, "y", h.instanceId).StructScan(instance)
 
+			// For future changes, please make sure that every branch and sub-branch within this switch creates at least
+			// one debug log event or returns with an error. This makes it easier to read the logs, since each time this
+			// function is called, it leaves a trace.
 			switch {
 			case errQuery == nil:
 				fields := []any{
@@ -336,9 +322,7 @@ func (h *HA) realize(
 					h.logger.Debugw("Preparing to take over HA as other instance's heartbeat has expired", fields...)
 				} else {
 					otherResponsible = true
-					if shouldLogRoutineEvents {
-						h.logger.Infow("Another instance is active", fields...)
-					}
+					h.logger.Logw(routineEventsLogLevel, "Another instance is active", fields...)
 				}
 
 			case errors.Is(errQuery, sql.ErrNoRows):
@@ -348,8 +332,8 @@ func (h *HA) realize(
 				if !h.responsible {
 					takeover = "no other instance is active"
 					h.logger.Debugw("Preparing to take over HA as no instance is active", fields...)
-				} else if h.responsible && shouldLogRoutineEvents {
-					h.logger.Debugw("Continuing being the active instance", fields...)
+				} else {
+					h.logger.Logw(routineEventsLogLevel, "Continuing being the active instance", fields...)
 				}
 
 			default:
@@ -376,6 +360,7 @@ func (h *HA) realize(
 				Icinga2EventHandlersEnabled:       s.EventHandlersEnabled,
 				Icinga2FlapDetectionEnabled:       s.FlapDetectionEnabled,
 				Icinga2PerformanceDataEnabled:     s.PerformanceDataEnabled,
+				IcingadbVersion:                   internal.Version.Version,
 			}
 
 			stmt, _ := h.db.BuildUpsertStmt(i)
@@ -435,18 +420,19 @@ func (h *HA) realize(
 			return nil
 		},
 		retry.Retryable,
+		// Keep the maximum backoff time small and not use backoff.DefaultBackoff here.
 		backoff.NewExponentialWithJitter(256*time.Millisecond, 3*time.Second),
 		retry.Settings{
 			// Intentionally no timeout is set, as we use a context with a deadline.
-			OnRetryableError: func(_ time.Duration, attempt uint64, err, lastErr error) {
-				if lastErr == nil || err.Error() != lastErr.Error() {
-					log := h.logger.Debugw
-					if attempt > 3 {
-						log = h.logger.Infow
-					}
-
-					log("Can't update or insert instance. Retrying", zap.Error(err))
+			OnRetryableError: func(elapsed time.Duration, attempt uint64, err, lastErr error) {
+				level := zap.DebugLevel
+				if attempt > 3 {
+					level = zap.WarnLevel
 				}
+				h.logger.Logw(level, "Can't update or insert instance. Retrying",
+					zap.Error(err),
+					zap.Duration("after", elapsed),
+					zap.Uint64("attempt", attempt))
 			},
 			OnSuccess: func(elapsed time.Duration, attempt uint64, lastErr error) {
 				if attempt > 1 {
