@@ -1,14 +1,18 @@
 package history
 
 import (
+	"github.com/icinga/icinga-go-library/redis"
+	"github.com/icinga/icinga-go-library/structify"
+	"github.com/icinga/icingadb/pkg/common"
+	"github.com/icinga/icingadb/pkg/contracts"
 	"github.com/icinga/icingadb/pkg/icingadb/v1/history"
-	"github.com/icinga/icingadb/pkg/structify"
-	"github.com/icinga/icingadb/pkg/types"
-	"github.com/redis/go-redis/v9"
 	"reflect"
 )
 
-var slaStateStructify = structify.MakeMapStructifier(reflect.TypeOf((*history.SlaHistoryState)(nil)).Elem(), "json")
+var slaStateStructify = structify.MakeMapStructifier(
+	reflect.TypeOf((*history.SlaHistoryState)(nil)).Elem(),
+	"json",
+	contracts.SafeInit)
 
 func stateHistoryToSlaEntity(entry redis.XMessage) ([]history.UpserterEntity, error) {
 	slaStateInterface, err := slaStateStructify(entry.Values)
@@ -17,7 +21,7 @@ func stateHistoryToSlaEntity(entry redis.XMessage) ([]history.UpserterEntity, er
 	}
 	slaState := slaStateInterface.(*history.SlaHistoryState)
 
-	if slaState.StateType != types.StateHard {
+	if slaState.StateType != common.HardState {
 		// only hard state changes are relevant for SLA history, discard all others
 		return nil, nil
 	}
