@@ -11,12 +11,11 @@ import (
 	"github.com/pkg/errors"
 	"os"
 	"path"
-	"time"
 )
 
 const (
-	expectedMysqlSchemaVersion    = 6
-	expectedPostgresSchemaVersion = 4
+	expectedMysqlSchemaVersion    = 7
+	expectedPostgresSchemaVersion = 5
 )
 
 // ErrSchemaNotExists implies that no Icinga DB schema has been imported.
@@ -60,7 +59,7 @@ func CheckSchema(ctx context.Context, db *database.DB) error {
 			return nil
 		},
 		retry.Retryable,
-		backoff.NewExponentialWithJitter(128*time.Millisecond, 1*time.Minute),
+		backoff.DefaultBackoff,
 		db.GetDefaultRetrySettings())
 	if err != nil {
 		return errors.Wrap(err, "can't check database schema version")
