@@ -3,14 +3,13 @@ package telemetry
 import (
 	"context"
 	"fmt"
+	"github.com/icinga/icinga-go-library/logging"
+	"github.com/icinga/icinga-go-library/periodic"
+	"github.com/icinga/icinga-go-library/redis"
+	"github.com/icinga/icinga-go-library/utils"
 	"github.com/icinga/icingadb/internal"
-	"github.com/icinga/icingadb/pkg/com"
 	"github.com/icinga/icingadb/pkg/icingaredis"
-	"github.com/icinga/icingadb/pkg/logging"
-	"github.com/icinga/icingadb/pkg/periodic"
-	"github.com/icinga/icingadb/pkg/utils"
 	"github.com/pkg/errors"
-	"github.com/redis/go-redis/v9"
 	"go.uber.org/zap"
 	"regexp"
 	"runtime/metrics"
@@ -78,18 +77,19 @@ func GetCurrentDbConnErr() (string, int64) {
 }
 
 // OngoingSyncStartMilli is to be updated by the main() function.
-var OngoingSyncStartMilli int64
-
-// LastSuccessfulSync is to be updated by the main() function.
-var LastSuccessfulSync com.Atomic[SuccessfulSync]
+var OngoingSyncStartMilli atomic.Int64
 
 var boolToStr = map[bool]string{false: "0", true: "1"}
 var startTime = time.Now().UnixMilli()
 
 // StartHeartbeat periodically writes heartbeats to Redis for being monitored by Icinga 2.
+// It returns an atomic pointer to SuccessfulSync,
+// which contains synchronisation statistics that the caller should update.
 func StartHeartbeat(
-	ctx context.Context, client *icingaredis.Client, logger *logging.Logger, ha ha, heartbeat *icingaredis.Heartbeat,
-) {
+	ctx context.Context, client *redis.Client, logger *logging.Logger, ha ha, heartbeat *icingaredis.Heartbeat,
+) *atomic.Pointer[SuccessfulSync] {
+	var syncStats atomic.Pointer[SuccessfulSync]
+	syncStats.Store(&SuccessfulSync{})
 	goMetrics := NewGoMetrics()
 
 	const interval = time.Second
@@ -100,8 +100,8 @@ func StartHeartbeat(
 	periodic.Start(ctx, interval, func(tick periodic.Tick) {
 		heartbeat := heartbeat.LastReceived()
 		responsibleTsMilli, responsible, otherResponsible := ha.State()
-		ongoingSyncStart := atomic.LoadInt64(&OngoingSyncStartMilli)
-		sync, _ := LastSuccessfulSync.Load()
+		ongoingSyncStart := OngoingSyncStartMilli.Load()
+		lastSync := syncStats.Load()
 		dbConnErr, dbConnErrSinceMilli := GetCurrentDbConnErr()
 		now := time.Now()
 
@@ -117,8 +117,8 @@ func StartHeartbeat(
 			"ha-responsible-ts":       strconv.FormatInt(responsibleTsMilli, 10),
 			"ha-other-responsible":    boolToStr[otherResponsible],
 			"sync-ongoing-since":      strconv.FormatInt(ongoingSyncStart, 10),
-			"sync-success-finish":     strconv.FormatInt(sync.FinishMilli, 10),
-			"sync-success-duration":   strconv.FormatInt(sync.DurationMilli, 10),
+			"sync-success-finish":     strconv.FormatInt(lastSync.FinishMilli, 10),
+			"sync-success-duration":   strconv.FormatInt(lastSync.DurationMilli, 10),
 		}
 
 		ctx, cancel := context.WithDeadline(ctx, tick.Time.Add(interval))
@@ -139,12 +139,14 @@ func StartHeartbeat(
 				silenceUntil = now.Add(time.Minute)
 			}
 
-			logw("Can't update own heartbeat", zap.Error(icingaredis.WrapCmdErr(cmd)))
+			logw("Can't update own heartbeat", zap.Error(redis.WrapCmdErr(cmd)))
 		} else {
 			lastErr = ""
 			silenceUntil = time.Time{}
 		}
 	})
+
+	return &syncStats
 }
 
 type goMetrics struct {
